@@ -20,6 +20,7 @@ limitations under the License.
 
 import re
 import os
+import urllib2
 import mechanize
 import BeautifulSoup
 
@@ -46,7 +47,7 @@ class Desire2Download(object):
 
         self.br = mechanize.Browser(factory=mechanize.RobustFactory())
         self.br.set_handle_refresh(mechanize._http.HTTPRefreshProcessor(), max_time=1)
- 
+
         self.br.open(self.ping_url).read()
 
     def retry(f):
@@ -56,14 +57,14 @@ class Desire2Download(object):
             while attempts < self.retries:
                 try:
                     return f(self, *args, **kargs)
-                except Exception as e:  #TODO: Important! should only catch timeouts
+                except Exception as e:  # TODO: Important! should only catch timeouts
                     attempts += 1
                     if attempts >= self.retries:
                         print "Timeout, out of retries."
                         raise(e)
                     print "Timeout, retrying..."
         return retry_it
-    
+
     @retry
     def login(self):
         print 'Logging In...'
@@ -75,18 +76,18 @@ class Desire2Download(object):
         if "Your userid and/or your password are incorrect" in response:
             raise AuthError("Your userid and/or your password are incorrect.")
         print 'Logged In'
-        
+
     def get_course_links(self):
         print 'Finding courses...'
         links = []
         urls = []
         for link in self.br.links():
-            matches = re.match('[A-Z]+ [0-9A-Z]{2,4} - [A-Z][a-z]+ 20[0-9]{2}', link.text)
+            matches = re.match('[A-Z]+ [0-9A-Z/\s]{2,15} - [A-Z][a-z]+ 20[0-9]{2}', link.text)
             if matches is not None and link.url not in urls:
                 links.append(link)
                 urls.append(link.url)
         return links
-        
+
     def convert_bytes(self, bytes):
         """
             Stolen from http://www.5dollarwhitebox.org/drupal/node/84
@@ -107,7 +108,7 @@ class Desire2Download(object):
         else:
             size = '%.2fb' % bytes
         return size
-    
+
     @retry
     def get_course_documents(self, link, course_name):
         """Produce a tree of documents for the course.
@@ -115,7 +116,7 @@ class Desire2Download(object):
         Args:
             link (str): A url to the course's page on d2l.
             course_name (str): The name of the course.
-        
+
         Returns:
             A dict representing a tree:
             {
@@ -147,7 +148,7 @@ class Desire2Download(object):
             depth = len(columns) - 1
             if len(path_to_root) >= depth:
                 path_to_root = path_to_root[:depth]
-           
+
             cell = row.find('td', 'd_gn')
             link = cell.find("a")
 
@@ -207,7 +208,7 @@ class Desire2Download(object):
             if e.errno != 17:
                 raise e
             pass
-            
+
         page = self.br.open(url).read()
         soup = BeautifulSoup.BeautifulSoup(page)
         url = soup.find('iframe')['src']
@@ -221,6 +222,12 @@ class Desire2Download(object):
             return
 
         url_path = url.split('?')[0]
+        if url_path.find('http') == 0:
+            # If this link is actually to the outside world, let it be.
+            # Technically this could be to ftp:// as well as many others.
+            clean_url = url_path
+        else:
+            clean_url = 'https://learn.uwaterloo.ca%s' % url_path
         clean_url = 'https://learn.uwaterloo.ca%s' % url_path
         clean_url = clean_url.replace(' ', '%20')
         file_name = os.path.split(url_path)[1]
@@ -230,10 +237,17 @@ class Desire2Download(object):
                 return
 
         path_and_filename = '%s/%s' % (path, file_name.strip('/'))
-        if os.path.isfile(path_and_filename):  ## TODO Can we make this smarter?
+        if os.path.isfile(path_and_filename):  # TODO Can we make this smarter?
             print ' - %s (Already Saved)' % path_and_filename
         else:
-            content = self.br.open_novisit(clean_url).read()
-            print ' + %s (%s)' % (path_and_filename, self.convert_bytes(len(content)))
-            with open(path_and_filename, 'w') as f:
-                f.write(content)
+            try:
+                content = self.br.open_novisit(clean_url).read()
+            except urllib2.HTTPError, e:
+                if e.code == 404:
+                    print " X File does not exist: %s" % file_name.strip('/')
+                else:
+                    print " X HTTP error %s for: %s" % (e.code, file_name.strip('/'))
+            else:
+                print ' + %s (%s)' % (path_and_filename, self.convert_bytes(len(content)))
+                with open(path_and_filename, 'w') as f:
+                    f.write(content)
